from django.utils import simplejson as json

from . import status
from .exceptions import HttpError


class BaseParser(object):
    " Base class for parsers. "

    media_type = None

    def __init__(self, resource):
        self.resource = resource

    @staticmethod
    def parse(request):
        return request.raw_post_data


class FormParser(BaseParser):
    " Parse user data from form data. "

    media_type = 'application/x-www-form-urlencoded'

    @staticmethod
    def parse(request):
        return dict(request.POST.iteritems())


class JSONParser(BaseParser):
    """ Parse user data from JSON.
        http://en.wikipedia.org/wiki/JSON
    """

    media_type = 'application/json'

    @staticmethod
    def parse(request):
        try:
            return json.loads(request.raw_post_data)
        except ValueError, e:
            raise HttpError('JSON parse error - %s' % str(e), status=status.HTTP_400_BAD_REQUEST)


class XMLParser(BaseParser):
    " Parse user data from XML. "

    media_type = 'application/xml'


try:
    from bson import BSON

    class BSONParser(BaseParser):
        """ Parse user data from bson.
            http://en.wikipedia.org/wiki/BSON
        """

        media_type = 'application/bson'

        @staticmethod
        def parse(request):
            try:
                return BSON(request.raw_post_data).decode()
            except ValueError, e:
                raise HttpError('BSON parse error - %s' % str(e), status=status.HTTP_400_BAD_REQUEST)

except ImportError:
    pass



from adrest.api import Api
from adrest.utils.auth import BaseAuthenticator
from adrest.utils.emitter import XMLEmitter
from adrest.views import ResourceView


class TestAuth(BaseAuthenticator):

    def authenticate(self, request=None):
        self.identifier = request.META.get('HTTP_AUTHORIZATION')
        return self.identifier


class TestResource(ResourceView):
    allowed_methods = 'GET', 'POST', 'PUT'
    model = 'rpc.test'

    def get(self, request, **resources):
        assert not 'error' in request.GET, "Custom error"
        return True


class RootResource(ResourceView):
    allowed_methods = 'GET', 'POST', 'PUT'
    model = 'rpc.root'


class ChildResource(ResourceView):
    allowed_methods = 'GET', 'POST', 'PUT'
    parent = RootResource
    model = 'rpc.child'


API = Api(api_rpc=True, emitters=(XMLEmitter,))
API.register(TestResource)
API.register(RootResource, authenticators=TestAuth)
API.register(ChildResource)



from StringIO import StringIO
from urlparse import urlparse

from django.core.urlresolvers import reverse
from django.db.models import Model
from django.test import TestCase, Client
from django.utils import simplejson
from django.utils.functional import curry


MULTIPART_CONTENT = 'multipart/form-data; boundary=BoUnDaRyStRiNg'


class AdrestClient(Client):

    def patch(self, path, data={}, content_type=MULTIPART_CONTENT, follow=False, **extra):
        " Send a resource to the server using PATCH. "

        patch_data = self._encode_data(data, content_type)
        parsed = urlparse(path)
        r = {
            'CONTENT_LENGTH': len(patch_data),
            'CONTENT_TYPE':   content_type,
            'PATH_INFO':      self._get_path(parsed),
            'QUERY_STRING':   parsed[4],
            'REQUEST_METHOD': 'PATCH',
            'wsgi.input':     FakePayload(patch_data),
        }
        r.update(extra)
        response = self.request(**r)
        if follow:
            response = self._handle_redirects(response, **extra)
        return response


class AdrestTestCase(TestCase):
    api = None

    def setUp(self):
        assert self.api, "API must be defined"
        self.client = AdrestClient()

    def reverse(self, resource, **kwargs):
        if isinstance(resource, basestring):
            url_name = resource
            assert self.api.resources.get(url_name), "Invalid resource name: %s" % url_name

        else:
            url_name = resource.meta.url_name

        kwargs = dict((k, getattr(v, "pk", v)) for k, v in kwargs.iteritems())
        name_ver = '' if not str(self.api) else '%s-' % str(self.api)
        return reverse('%s-%s%s' % (self.api.prefix, name_ver, url_name), kwargs=kwargs)

    def get_resource(self, resource, method='get', data=None, key=None, headers=None, **kwargs):
        uri = self.reverse(resource, **kwargs)
        method = getattr(self.client, method)
        if isinstance(key, Model):
            key = key.key
        headers = dict() if headers is None else headers
        headers['HTTP_AUTHORIZATION'] = key or headers.get('HTTP_AUTHORIZATION')
        return method(uri, data=data or dict(), **headers)

    def rpc(self, data, callback=None, headers=None, key=None, **kwargs):
        data = dict(payload=simplejson.dumps(data))
        if callback:
            data['callback'] = callback

        # JSONP not support headers
        if headers and headers.get('HTTP_ACCEPT') == 'text/javascript':
            headers = dict(HTTP_ACCEPT='text/javascript')
            key = None

        return self.get_resource('rpc', data=data, headers=headers, key=key, **kwargs)

    put_resource = curry(get_resource, method='put')
    post_resource = curry(get_resource, method='post')
    patch_resource = curry(get_resource, method='patch')
    delete_resource = curry(get_resource, method='delete')


class FakePayload(object):
    """
    A wrapper around StringIO that restricts what can be read since data from
    the network can't be seeked and cannot be read outside of its content
    length. This makes sure that views can't do anything under the test client
    that wouldn't work in Real Life.
    """
    def __init__(self, content):
        self.__content = StringIO(content)
        self.__len = len(content)

    def read(self, num_bytes=None):
        if num_bytes is None:
            num_bytes = self.__len or 0
        assert self.__len >= num_bytes, "Cannot read more than the available bytes from the HTTP incoming data."
        content = self.__content.read(num_bytes)
        self.__len -= num_bytes
        return content



import base64

from django.contrib.auth import authenticate
from django.middleware.csrf import CsrfViewMiddleware

from adrest import models


class BaseAuthenticator(object):
    " Abstract base authenticator "

    def __init__(self, resource):
        self.resource = resource
        self.identifier = ''

    def authenticate(self, request=None):
        self.identifier = self.get_identifier(request)
        return self.identifier

    def get_identifier(self, request=None):
        return self.identifier

    @staticmethod
    def test_rights(resources, request=None):
        return True

    @staticmethod
    def get_fields():
        return []


class AnonimousAuthenticator(BaseAuthenticator):
    " Anonymous access "

    @staticmethod
    def get_identifier(request):
        return request.META.get('REMOTE_ADDR', 'anonymous')


class _UserAuthenticator(BaseAuthenticator):
    " Abstract class for user authentication "

    def __init__(self, resource=None):
        super(_UserAuthenticator, self).__init__(resource)
        self.user = None

    def get_identifier(self, request):
        if self.user and self.user.is_active:
            self.identifier = self.user.username
        return self.identifier


class UserLoggedInAuthenticator(_UserAuthenticator):
    " Authorization by session "

    def authenticate(self, request=None):
        if getattr(request, 'user', None):
            resp = CsrfViewMiddleware().process_view(request, None, (), {})
            if resp is None:  # csrf passed
                return self.get_identifier()

        return False


class BasicAuthenticator(_UserAuthenticator):
    " HTTP Basic authentication "

    def authenticate(self, request=None):
        if 'HTTP_AUTHORIZATION' in request.META:
            auth = request.META['HTTP_AUTHORIZATION'].split()
            if len(auth) == 2 and auth[0].lower() == "basic":
                uname, passwd = base64.b64decode(auth[1]).split(':')
                self.user = authenticate(username=uname, password=passwd)
                return self.get_identifier(request)

        return False


class UserAuthenticator(_UserAuthenticator):
    " Authorization by login and password "

    username_fieldname = 'username'
    password_fieldname = 'password'

    def authenticate(self, request=None):
        try:
            username = request.REQUEST.get(self.username_fieldname)
            password = request.REQUEST.get(self.password_fieldname)
            self.user = request.user = authenticate(username=username, password=password)
            return self.get_identifier(request)

        except KeyError:
            return False

        return False

    @classmethod
    def get_fields(cls):
        return [(cls.username_fieldname, dict(required=True)), (cls.password_fieldname, dict(required=True))]


class AccessKeyAuthenticator(_UserAuthenticator):
    " Authorization by API key "

    def authenticate(self, request=None):
        """ Authenticate user using AccessKey from HTTP Header or GET params.
        """
        try:
            access_key = request.META.get('HTTP_AUTHORIZATION') or request.REQUEST['key']
            api_key = models.AccessKey.objects.get(key=access_key)
            self.user = request.user = api_key.user
            return self.get_identifier(request)

        except(KeyError, models.AccessKey.DoesNotExist):
            return False

        return False



def as_tuple(obj):
    " Given obj return a tuple "

    if not obj:
        return tuple()

    if isinstance(obj, (tuple, set, list)):
        return tuple(obj)

    return obj,


def gen_url_name(resource):
    " URL name for resource class generator. "
    if resource.parent:
        yield resource.parent.meta.url_name

    if resource.prefix:
        yield resource.prefix

    for p in resource.url_params:
        yield p

    yield resource.meta.name


def gen_url_regex(resource):
    " URL regex for resource class generator. "
    for r in resource.meta.parents:
        if r.url_regex:
            yield r.url_regex.rstrip('/$').lstrip('^')
        else:
            yield '%(name)s/(?P<%(name)s>[^/]+)' % dict(
                    name=r.meta.name)

    for p in resource.url_params:
        yield '%(name)s/(?P<%(name)s>[^/]+)' % dict(name=p)

    if resource.prefix:
        yield resource.prefix

    yield '%(name)s/(?:(?P<%(name)s>[^/]+)/)?' % dict(name=resource.meta.name)


def fix_request(request):
    methods = "PUT", "PATCH"

    if request.method in methods and not getattr(request, request.method, None):

        if hasattr(request, '_post'):
            del(request._post)
            del(request._files)

        if hasattr(request, '_request'):
            del(request._request)

        request.method, method = "POST", request.method
        setattr(request, method, request.POST)
        request.method = method

    return request



from .resources import AuthorResource, BookPrefixResource, ArticleResource, SomeOtherResource, CustomResource, BSONResource
from adrest.api import Api
from adrest.utils.auth import AnonimousAuthenticator, AccessKeyAuthenticator, UserAuthenticator
from adrest.utils.emitter import XMLTemplateEmitter, JSONEmitter, BSONEmitter
from adrest.utils.parser import BSONParser


class CustomUserAuth(UserAuthenticator):
    username_fieldname = 'nickname'


API = Api(version=(1, 0, 0), emitters=(XMLTemplateEmitter, JSONEmitter), api_prefix='main')

API.register(AuthorResource,
             authenticators=(CustomUserAuth, AnonimousAuthenticator))
API.register(BookPrefixResource)
API.register(CustomResource)
API.register(ArticleResource, authenticators=AccessKeyAuthenticator)
API.register(SomeOtherResource, url_name='test', url_regex='test/mem/$')
API.register(BSONResource, parsers=(BSONParser,), emitters=(BSONEmitter,))



#!/usr/bin/env python
import os

from setuptools import setup, find_packages

from adrest import version, PROJECT, LICENSE


PACKAGE_DATA = []

for folder in ['templates']:
    for root, dirs, files in os.walk(os.path.join(PROJECT, folder)):
        for filename in files:
            PACKAGE_DATA.append("%s/%s" % (root[len(PROJECT) + 1:], filename))


def read(fname):
    try:
        return open(os.path.join(os.path.dirname(__file__), fname)).read()
    except IOError:
        return ''


def run_tests():
    """Run tests without install library into python path
    """
    from tests import run_tests
    return run_tests()


META_DATA = dict(
    name=PROJECT,
    version=version,
    LICENSE=LICENSE,
    description=read('DESCRIPTION'),
    long_description=read('README.rst'),
    platforms=('Any'),

    author='Kirill Klenov',
    author_email='horneds@gmail.com',
    url=' http://github.com/klen/adrest',

    packages=find_packages(),
    package_data = {'': PACKAGE_DATA},

    install_requires = ('mimeparse', 'Django>=1.3.0'),
    test_suite = '__main__.run_tests',
    tests_require = 'pymongo'
)


if __name__ == "__main__":
    setup(**META_DATA)



from django.http import HttpResponse


class SerializedHttpResponse(HttpResponse):
    """ Response has will be serialized.
        Django http response will be returned as is.
    """

    def __init__(self, content='', *args, **kwargs):
        self.response = content
        super(SerializedHttpResponse, self).__init__(content, *args, **kwargs)
